<?php

/**
 * Pallapay     Payment Gateway
 *
 * Provides a Pallapay Payment Gateway; mainly for testing purposes.
 * We load it later to ensure WC is loaded first since we're extending it.
 *
 * @class        WC_Pallapay_Gateway
 * @extends        WC_Payment_Gateway
 * @version        1.0.0
 */
add_action('plugins_loaded', 'wc_pallapay_gateway_init', 11);

function wc_pallapay_gateway_init()
{

    class WC_Pallapay_Gateway extends WC_Payment_Gateway
    {

        /**
         * Constructor for the gateway.
         */
        public function __construct()
        {

            $this->id = 'pallapay_gateway';
            $this->has_fields = false;
            $this->method_title = __('Pallapay', 'wpg');
            $this->method_description = __('Pallapay Payment Gateway', 'wpg');
            $this->payment_url = 'https://www.pallapay.com/api/v1/request/createOrder';

            /* IPN URL */
            $this->ipnlink = add_query_arg('wc-api', 'wc_pallapay', trailingslashit(get_home_url()));
            // Load the settings.
            $this->init_form_fields();
            $this->init_settings();

            // Define user set variables
            $this->title = $this->get_option('title');
            $this->description = $this->get_option('description');
            $this->merchant_id = $this->get_option('merchant_id');
            $this->merchant_password = $this->get_option('merchant_password');

            add_action('init', array(&$this, 'wpg_check_pallapay_response'));
            add_action('woocommerce_api_' . strtolower(get_class($this)), array($this, 'wpg_check_pallapay_response')); //update for woocommerce >2.0
            // Actions
            if (version_compare(WOOCOMMERCE_VERSION, '2.0.0', '>=')) {
                add_action('woocommerce_update_options_payment_gateways_' . $this->id, array(&$this, 'process_admin_options')); //update for woocommerce >2.0
            } else {
                add_action('woocommerce_update_options_payment_gateways', array(&$this, 'process_admin_options')); // WC-1.6.6
            }

            add_action('init', array(&$this, 'check_payuindia_response'));
            add_action('woocommerce_api_' . strtolower(get_class($this)), array($this, 'check_payuindia_response')); //update for woocommerce >2.0
            // Customer Emails
            add_action('woocommerce_email_before_order_table', array($this, 'email_instructions'), 10, 3);
            add_action('woocommerce_receipt_pallapay_gateway', array(&$this, 'receipt_page'));
        }

        function receipt_page($order)
        {
            echo '<p><strong>' . __('Thank you for your order.', 'wpg') . '</strong><br/>' . __('The payment page will open soon.', 'wpg') . '</p>';
            echo $this->generate_pallapay_form($order);
        }

        //END-receipt_page

        /**
         * Generate button link
         * */
        function get_order_item_names($order)
        {
            $item_names = $item_id = $result = array();

            foreach ($order->get_items() as $item) {
                $item_name = $item->get_name();
                $item_id [] = $item->get_ID();

                $item_meta = strip_tags(
                    wc_display_item_meta(
                        $item, array(
                            'before' => '',
                            'separator' => ', ',
                            'after' => '',
                            'echo' => false,
                            'autop' => false,
                        )
                    )
                );

                if ($item_meta) {
                    $item_name .= ' (' . $item_meta . ')';
                }

                $item_names[] = $item_name;
            }
            $result['itemname'] = implode(', ', $item_names);
            $result['itemid'] = implode(', ', $item_id);
            return apply_filters('pallapay_order_get_order_item_names', $result, $order);
        }

        function generate_pallapay_form($order_id)
        {
            global $woocommerce;
            $redirect_url = get_permalink($this->redirect_page);
            if (version_compare(WOOCOMMERCE_VERSION, '2.0.0', '>=')) {
                $redirect_url = add_query_arg('wc-api', get_class($this), $redirect_url);
            }

            $order = new WC_Order($order_id);
            $item_info = $this->get_order_item_names($order);
            $all_items_name = $item_info['itemname'];
            $all_items_number = $item_info['itemid'];

            $paramet = json_encode([
                'order' => $order_id,
                'merchant' => $this->merchant_id,
                'item_name' => $all_items_name,
                'item_number' => $all_items_number,
                'amount' => $order->get_total(),
                'currency' => get_woocommerce_currency(),
                'custom' => $order->get_customer_note(),
                'first_name' => $order->get_billing_first_name(),
                'last_name' => $order->get_billing_last_name(),
                'email' => $order->get_billing_email(),
                'phone' => $order->get_billing_phone(),
                'address' => $order->get_billing_address_1(),
                'city' => $order->get_billing_city(),
                'state' => $order->get_billing_state(),
                'country' => $order->get_billing_country(),
                'postalcode' => $order->get_billing_postcode(),
                'success_url' => $order->get_checkout_order_received_url(),
                'notify_url' => $redirect_url,
                'fail_link' => $order->get_cancel_order_url()
            ]);

            $curlCli = curl_init($this->payment_url);
            curl_setopt($curlCli, CURLOPT_HTTPHEADER, [
                'Content-Type: application/json'
            ]);
            curl_setopt($curlCli, CURLOPT_RETURNTRANSFER, TRUE);
            curl_setopt($curlCli, CURLOPT_SSL_VERIFYPEER, TRUE);
            curl_setopt($curlCli, CURLOPT_CUSTOMREQUEST, 'POST');
            curl_setopt($curlCli, CURLOPT_POST, TRUE);
            curl_setopt($curlCli, CURLOPT_POSTFIELDS, $paramet);
            $result = curl_exec($curlCli);
            curl_close($curlCli);
            $resultData = json_decode($result, TRUE);

            if ($resultData['status'] == 1) {
                //redirect user to payment link.
                sleep(3);
                wp_redirect($resultData['result']['redirect_to_url']);
                exit;
            } else {
                $debug_text = "";
                if (WP_DEBUG) {
                    $debug_text = "<div style='padding-top: 10px; font-size: 1rem'>RESPONSE: " . is_array($resultData) ? "Response: " . json_encode($resultData) : "NOTHING" . "</div>";
                }
                echo "<div style='background-color: #f6f6f6; padding: 20px 30px; border-radius: 3px; border: 1px solid #d50000; margin-bottom: 25px; color: #d50000; font-size: 1.4rem'>Error while creating payment. {$debug_text}</div>";
            }
        }

        /**
         * Initialize Gateway Settings Form Fields
         */
        public function init_form_fields()
        {

            $this->form_fields = apply_filters('wc_pallapay_form_fields', array(
                'enabled' => array(
                    'title' => __('Enable/Disable', 'wpg'),
                    'type' => 'checkbox',
                    'label' => __('Enable Pallapay Payment', 'wpg'),
                    'default' => 'yes'
                ),
                'title' => array(
                    'title' => __('Title', 'wpg'),
                    'type' => 'text',
                    'description' => __('This controls the title for the payment method the customer sees during checkout.', 'wpg'),
                    'default' => __('Crypto Currencies', 'wpg'),
                    'desc_tip' => true,
                ),
                'description' => array(
                    'title' => __('Description', 'wpg'),
                    'type' => 'textarea',
                    'description' => __('Payment method description that the customer will see on your checkout.', 'wpg'),
                    'default' => __('Pay Securely With Crypto Currencies Via Pallapay.', 'wpg'),
                    'desc_tip' => true,
                ),
                'merchant_id' => array(
                    'title' => __('Merchant ID', 'wpg'),
                    'type' => 'text',
                    'description' => __('If you do not have merchant ID, sign up in pallapay.com and get one.', 'wpg'),
                    'default' => '',
                    'desc_tip' => true,
                ),
                'merchant_password' => array(
                    'title' => __('Merchant Password', 'wpg'),
                    'type' => 'password',
                    'description' => __('Merchant password is required to validate IPN notifications.', 'wpg'),
                    'default' => '',
                    'desc_tip' => true,
                ),
            ));
        }

        /**
         * Add content to the WC emails.
         *
         * @access public
         * @param WC_Order $order
         * @param bool $sent_to_admin
         * @param bool $plain_text
         */
        public function email_instructions($order, $sent_to_admin, $plain_text = false)
        {

            if ($this->instructions && !$sent_to_admin && $this->id === $order->payment_method && $order->has_status('on-hold')) {
                echo wpautop(wptexturize($this->instructions)) . PHP_EOL;
            }
        }

        /**
         * Process the payment and return the result
         *
         * @param int $order_id
         * @return array
         */
        public function process_payment($order_id)
        {

            global $woocommerce;
            $order = new WC_Order($order_id);

            $checkout_payment_url = $order->get_checkout_payment_url(true);

            return array(
                'result' => 'success',
                'redirect' => add_query_arg(
                    'order',
                    $order->get_id(),
                    add_query_arg(
                        'key',
                        $order->get_order_key(),
                        $checkout_payment_url
                    )
                )
            );
        }

        public function wpg_check_pallapay_response()
        {
            global $woocommerce;
            /* Change IPN URL */
            if (isset($_REQUEST['hash']) && isset($_REQUEST['order'])) {
                $order_id = $_REQUEST['order'];
                if ($order_id != '') {
                    try {
                        $order = new WC_Order($order_id);
                        $hash = $_REQUEST['hash'];
                        $status = $_REQUEST['status'];
                        $trans_authorised = false;

                        if ($order->get_status() !== 'completed' && $status == 'Confirmed') {
                            $status = strtolower($status);
                            $requestHash = strtoupper(md5($_REQUEST['total'] . ':' . $this->merchant_password . ':' . $_REQUEST['date'] . ':' . $_REQUEST['id_transfer']));
                            if ($status == 'confirmed' && $_REQUEST['hash'] == $requestHash) {
                                $trans_authorised = true;
                                $this->msg['message'] = "Thank you for the order. Your account has been charged and your transaction is successful.";
                                $this->msg['class'] = 'success';
                                $order->add_order_note('Pallapay payment successful.<br/>Pallapay Transaction ID: ' . $_REQUEST['id_transfer']);
                                $order->payment_complete();
                                $woocommerce->cart->empty_cart();
                            } else {
                                $this->msg['class'] = 'error';
                                $this->msg['message'] = "The transaction has been declined.";
                                $order->add_order_note('Transaction Fail');
                            }
                        }
                    } catch (Exception $e) {
                        $msg = "Error";
                    }
                }

                if (function_exists('wc_add_notice')) {
                    wc_add_notice($msg['message'], $msg['class']);
                } else {
                    if ('success' == $msg['class']) {
                        $woocommerce->add_message($msg['message']);
                    } else {
                        $woocommerce->add_error($msg['message']);
                    }
                    $woocommerce->set_messages();
                }

                // @see: https://wordpress.org/support/topic/enabling-default-woocommerce-redirects/#post-9728440
                if ('success' == $this->msg['class']) {
                    if ('' == $this->redirect_page || 0 == $this->redirect_page) {
                        $redirect_url = $this->get_return_url($order);
                    } else {
                        $redirect_url = get_permalink($this->redirect_page);
                    }
                } else {
                    $redirect_url = wc_get_checkout_url();
                }

                wc_print_notices();

                wp_redirect($redirect_url);
                exit;
            }
        }

    }

    // end \WC_Gateway_Bring class
}